/*
    rtctime: read/set real time clock, including seconds
    93/09/25 John H. DuBois III (john@armory.com)
*/

#include <sys/rtc.h>
#include <fcntl.h>
#include <stdio.h>

/* convert 2-char BCD value to unsigned integer & return it */
#define BCD2int(Val) (((unsigned char) Val >> 4) * 10 + (Val & 0xF))
/* convert unsigned int to 2-char BCD value & return it */
#define int2BCD(Val) ((((unsigned) Val / 10) << 4) | ((unsigned) Val % 10))
char *Comment = "@(#) rtctime 1.1 96/07/03";

main(argc,argv)
int argc;
char *argv[];
{
    struct rtc_t Buf;
    int Mode;
    static char Weekdays[7][4] = { "Sun","Mon","Tue","Wed","Thu","Fri","Sat" };

    if (argc >= 2 && !strcmp(argv[1],"-h")) {
	fputs(
"Usage: rtctime [-h] [[[hours] minutes] seconds]\n"\
"If no arguments are given, the date and time are read from the real time\n"\
"clock and printed.  If a time is given, it is used to set the real time\n"\
"clock.  Either seconds, or minutes and seconds, or hours, minutes, and\n"\
"seconds may be given.  Time components not given are not changed.\n",stdout);
	exit(0);
    }
    if (RTC_IO(O_RDONLY,RTCRTIME,&Buf))
	exit(1);
    if (argc < 2) {
	printf("%02d-%02d-%02d %s %02d:%02d:%02d\n",
	BCD2int(Buf.rtc_yr),
	BCD2int(Buf.rtc_mon),BCD2int(Buf.rtc_dom),Weekdays[Buf.rtc_dow],
	BCD2int(Buf.rtc_hr),BCD2int(Buf.rtc_min),BCD2int(Buf.rtc_sec));
	exit(0);
    }
    else
	exit(SetRTC(&Buf,argc - 1, argv + 1));
}

SetRTC(Buf,argc,argv)
struct rtc_t *Buf;
int argc;
char **argv;
{
    int Vals[3];
    char *ptr;
    char **Arg = argv - (3 - argc);
    int i;
    static int MaxVals[3] = { 23, 59, 59 };
    static char *ValNames[3] = { "hours", "minutes", "seconds" };
    
    for (i = 0; i < 3; i++)
	if ((2 - i) < argc) {
	    Vals[i] = (int) strtol(Arg[i],&ptr,10);
	    /* If value is the null string, or any of the string was
	       not part of a legal integer, or result was negative, fail */
	    if (!Arg[i] || *ptr || Vals[i] < 0) {
		fprintf(stderr,
		"rtc_time: Illegal value \"%s\": Must be a positive integer.\n",
		Arg[i]);
		return(1);
	    }
	}
	else
	    Vals[i] = -1;

    for (i = 0; i < 3; i++)
	if (Vals[i] > MaxVals[i]) {
	    fprintf(stderr,
	    "rtc_time: %d: Value of %s out of range (should be 0-%d).\n",
	    Vals[i],ValNames[i],MaxVals[i]);
	    return(1);
	}
    if (Vals[0] != -1)
	Buf->rtc_hr = int2BCD(Vals[0]);
    if (Vals[1] != -1)
	Buf->rtc_min = int2BCD(Vals[1]);
    if (Vals[2] != -1)
	Buf->rtc_sec = int2BCD(Vals[2]);

    return (RTC_IO(O_WRONLY,RTCSTIME,Buf));
}

RTC_IO(Mode,Cmd,Buf)
int Mode;
int Cmd;
struct rtc_t *Buf;
{
    int rtc_fd;

    if ((rtc_fd = open("/dev/rtc",Mode)) == -1) {
	perror("rtctime: could not open /dev/rtc");
	return 1;
    }
    if (ioctl(rtc_fd,Cmd,Buf) == -1) {
	perror("rtctime: ioctl on /dev/rtc failed");
	return 1;
    }
    close(rtc_fd);
    return 0;
}
